@@ -58,13 +58,23 @@ filter_new (const gchar                    *pattern,
         new->patterns = search_util_get_patterns (pattern);
         new->fnmatch_flags = 0;
 	new->options = options;
-        if (options & GNOME_VFS_DIRECTORY_FILTER_IGNORECASE)
+        if ((options & GNOME_VFS_DIRECTORY_FILTER_IGNORECASE) == GNOME_VFS_DIRECTORY_FILTER_IGNORECASE)
                 new->fnmatch_flags |= FNM_CASEFOLD;
 
         return new;
 }
 
 
+static void
+filter_set_options (Filter                         *filter,
+		    GnomeVFSDirectoryFilterOptions  options)
+{
+	filter->options = options;
+        if ((options & GNOME_VFS_DIRECTORY_FILTER_IGNORECASE) == GNOME_VFS_DIRECTORY_FILTER_IGNORECASE)
+                filter->fnmatch_flags |= FNM_CASEFOLD;
+}
+
+
 static void
 filter_destroy (Filter *filter)
 {
@@ -90,11 +100,11 @@ filter_apply (Filter     *filter,
 	file_name = file_name_from_path (name);
 
 	if ((filter->options & GNOME_VFS_DIRECTORY_FILTER_NODOTFILES)
-	    && ((name[0] == '.') || (strstr (name, "/.") != NULL)))
+	    && ((file_name[0] == '.') || (strstr (file_name, "/.") != NULL)))
 		return FALSE;
 
 	if ((filter->options & GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES)
-	    && (name[strlen (name) - 1] == '~'))
+	    && (file_name[strlen (file_name) - 1] == '~'))
 		return FALSE;
 
 	utf8_name = g_filename_to_utf8 (file_name, -1, NULL, NULL, NULL);
@@ -193,6 +203,27 @@ wc_visit_cb (const gchar      *rel_path,
 }
 
 
+static gboolean
+pattern_present (char       **patterns,
+		 const char  *pattern)
+{
+	gboolean found = FALSE;
+	int      i;
+       
+	if ((patterns == NULL) || (patterns[0] == NULL))
+		return FALSE;
+	
+	if (pattern == NULL)
+		return FALSE;
+	
+	for (i = 0; (patterns[i] != NULL) && !found; i++) 
+		if (g_utf8_collate (patterns[i], pattern) == 0) 
+			found = TRUE;
+	
+	return found;
+}
+
+
 GList *
 get_wildcard_file_list (const char  *directory, 
 			const char  *filter_pattern, 
@@ -217,15 +248,16 @@ get_wildcard_file_list (const char  *directory,
 
 	/* file filter */
 
+	data->filter = filter_new (filter_pattern, GNOME_VFS_DIRECTORY_FILTER_DEFAULT);
+
 	filter_options = GNOME_VFS_DIRECTORY_FILTER_DEFAULT;
-	if (no_backup_files)
+	if (no_backup_files && !pattern_present (data->filter->patterns, "*~"))
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES;
-	if (no_dot_files)
+	if (no_dot_files && !pattern_present (data->filter->patterns, ".*"))
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NODOTFILES;
 	if (ignorecase)
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_IGNORECASE;
-
-	data->filter = filter_new (filter_pattern, filter_options);
+	filter_set_options (data->filter, filter_options);
 
 	/* info options */
 
@@ -361,7 +393,6 @@ get_directory_file_list (const char *directory,
 			 const char *base_dir)
 {
 	DirSearchData *                data;
-	GnomeVFSDirectoryFilterOptions filter_options;
 	GnomeVFSResult                 result;
 	GnomeVFSFileInfoOptions        info_options; 
 	GnomeVFSDirectoryVisitOptions  visit_options;
@@ -371,9 +402,9 @@ get_directory_file_list (const char *directory,
 
 	/* file filter */
 
-	filter_options = (GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES
-			  | GNOME_VFS_DIRECTORY_FILTER_NODOTFILES);
-	data->filter = filter_new ("*", filter_options);
+	data->filter = filter_new ("*", 
+				   (GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES
+				    | GNOME_VFS_DIRECTORY_FILTER_NODOTFILES));
 
 	/* options. */
 
@@ -562,7 +593,7 @@ path_list_async_new (const char       *uri,
 		     PathListDoneFunc  f,
 		     gpointer          data)
 {
-	GnomeVFSAsyncHandle *handle;
+	GnomeVFSAsyncHandle *handle = NULL;
 	PathListData        *pli;
 	char                *escaped;
 	PathListHandle      *pl_handle;
@@ -819,9 +850,9 @@ visit_dir_async (const char       *directory,
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NOBACKUPFILES;
 	if (no_dot_files)
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_NODOTFILES;
-	if (ignorecase)
+	if (ignorecase) 
 		filter_options |= GNOME_VFS_DIRECTORY_FILTER_IGNORECASE;
-
+	
 	if (filter_pattern != NULL)
 		vdd->filter = filter_new (filter_pattern, filter_options);
 
@@ -893,7 +924,7 @@ get_wildcard_file_list_cb (GList *files, gpointer data)
 		for (scan = files; scan; scan = scan->next) {
 			char *full_path = scan->data;
 			
-			if (strncmp (full_path, gfl_data->directory, base_len) == 0) {
+			if (path_in_path (gfl_data->directory, full_path)) {
 				char *rel_path = g_strdup (full_path + base_len + 1);
 				rel_files = g_list_prepend (rel_files, rel_path);
 			}
@@ -951,12 +982,16 @@ get_directory_file_list_cb (GList *files, gpointer data)
 
 	if (gfl_data->base_dir != NULL) {
 		GList *scan;
-		int    base_len = strlen (gfl_data->base_dir);
+		int    base_len;
+		
+		base_len = 0;
+		if (strcmp (gfl_data->base_dir, "/") != 0)
+			base_len = strlen (gfl_data->base_dir);
 
 		for (scan = files; scan; scan = scan->next) {
 			char *full_path = scan->data;
 			
-			if (strncmp (full_path, gfl_data->base_dir, base_len) == 0) {
+			if (strncmp (gfl_data->base_dir, full_path, base_len) == 0) {
 				char *rel_path = g_strdup (full_path + base_len + 1);
 				rel_files = g_list_prepend (rel_files, rel_path);
 			}
@@ -990,15 +1025,18 @@ get_directory_file_list_async (const char       *directory,
 	gfl_data->done_func = done_func;
 	gfl_data->done_data = done_data;
 
-	path = g_strconcat (base_dir, "/", directory, NULL);
-	
+	if (strcmp (base_dir, "/") == 0)
+        	path = g_strconcat (base_dir, directory, NULL);
+	else
+		path = g_strconcat (base_dir, "/", directory, NULL);
+
 	handle = visit_dir_async (path,
 				  "*",
 				  TRUE,
 				  TRUE,
 				  TRUE,
-				  TRUE,
-				  TRUE,
+				  FALSE,
+				  FALSE,
 				  FALSE,
 				  get_directory_file_list_cb,
 				  gfl_data);
