@@ -152,9 +152,9 @@ process_line (char     *line,
 		} else {
 			fdata->size = atoll (fields[0]);
 			fdata->modified = mktime_from_string (fields[3], fields[4]); 
-			g_strfreev (fields);
 			comm->file_list = g_list_prepend (comm->file_list, fdata);
 		}
+		g_strfreev (fields);
 
 		rar_comm->fdata = NULL;
 	}
@@ -247,6 +247,9 @@ fr_command_rar_add (FRCommand     *comm,
 	}
 
 	add_password_arg (comm, password, FALSE);
+
+	/* disable percentage indicator */
+	fr_process_add_arg (comm->process, "-idp");
 	
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--"); 
@@ -310,6 +313,9 @@ fr_command_rar_extract (FRCommand  *comm,
 		fr_process_add_arg (comm->process, "-ep");
 	
 	add_password_arg (comm, password, TRUE);
+
+	/* disable percentage indicator */
+	fr_process_add_arg (comm->process, "-idp");
 	
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--"); 
@@ -341,7 +347,10 @@ fr_command_rar_test (FRCommand   *comm,
 	fr_process_add_arg (comm->process, "t");
 	
 	add_password_arg (comm, password, TRUE);
-	
+
+	/* disable percentage indicator */
+	fr_process_add_arg (comm->process, "-idp");
+
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 	
